'
' Create a Compare Interrupt using the Special Event mechanism
' Compares the 16-bit value of Timer3 with the contents of 16-bit registers CCPR2L\H
' Triggers the interrupt when Timer3 reaches the value held in CCPR2L\H
' Timer3 is automatically reset in hardware
'
' Uses the interrupt manager routines for easier use of high priority interrupts
'
$define Handle_CCP2()                               ' Handle the CCP2 interrupt

    Include "Interrupt Manager.inc"                 ' Load the interrupt manager routines into the program
    Include "Timers.inc"                            ' Include the Timer macros into the program

    Dim tTimerReady As Bit                          ' True if the millisecond variable has been incremented
    Dim dMilliSeconds As Dword                      ' Holds the incrementing 32-bit value

    Dim wCCPR2 As CCPR2L.Word                       ' Combine CCP2L\H into a 16-bit register

'--------------------------------------------------------------------------------------------------
    GoTo Main                                       ' Jump over any subroutines
'--------------------------------------------------------------------------------------------------
' Special Event on CCP2 Interrupt Handler
' Increment the 32-bit variable dMilliSeconds
' Sets the flag tTimerReady when the counter has incremented.
' This must be reset in the main program
' The interrupt is configured to trigger every 1 millisecond
'
ISR_CCP2(Start)                                     ' Indicate the start of the CCP2 interrupt handler
    Inc dMilliSeconds                               ' Increment the counting variable
    tTimerReady = True                              ' Indicate that the millisecond timer has been updated
ISR_CCP2(Exit)                                      ' Exit the CCP2 interrupt handler

'--------------------------------------------------------------------------------------------------
' The main program loop starts here
'
Main:
    CCP2CON = %00001011                             ' Compare mode: trigger special event, reset timer, start A/D conversion on CCP2 match (CCP2IF bit is set)
    $define cPrescalerValue 8                       ' Set the timer's prescaler value

$if cPrescalerValue = 8
    $define T3_Prescaler T3_PS_1_8
$elseif cPrescalerValue = 4
    $define T3_Prescaler T3_PS_1_4
$elseif cPrescalerValue = 2
    $define T3_Prescaler T3_PS_1_2
$elseif cPrescalerValue = 1
    $define T3_Prescaler T3_PS_1_1
$endif
'
' Setup Timer3:
'           Clear TMPR3L\H
'           Interrupt disabled
'           16-bit read-write mode
'           Internal clock source
'           1:n prescaler
'           Don't sync external clock input
'           Make Timer3 the source for a compare interrupt
'
    OpenTimer3(TIMER_INT_OFF & T3_16BIT_RW & T3_Prescaler & T3_SYNC_EXT_OFF & T3_SOURCE_INT & T3_SOURCE_CCP)
    $define cMicroSeconds 1000                      ' Interrupt rate (in MicroSeconds)
'
' Calculate the value to place into the CCPR2L\H registers in order to achieve a certain interrupt rate (in us)
'
    $define cCCPR_Value $eval (cMicroSeconds / cPrescalerValue) * (_xtal / 4)

$if cCCPR_Value > 65535
    $error "Value too large for interrupt duration"
$elseif cCCPR_Value = 0
    $error "Value too small for interrupt duration"
$endif

    dMilliSeconds = 0                               ' Reset the millisecond counter
    wCCPR2 = cCCPR_Value                            ' Load CCPR2L\H with the value to trigger an interrupt at a certain duration
    Int_CCP2(Enable)                                ' Enable the CCP2 interrupt
    Int_Global(Enable)                              ' Enable Global Interrupts
'
' Display the millisecond counter running in the background
'
    While 1 = 1                                     ' Create an endless loop
        If tTimerReady = True Then                  ' Has the millisecond timer incremented?
            tTimerReady = False                     ' Yes. So reset the flag
            HRSOut Dec dMilliSeconds, 13            ' Transmit the ASCII value of the counter
        EndIf
    Wend                                            ' Do it forever

